#include "tuner.h"
#include "ui_tuner.h"
#include "ffthandler.h"
#include "micmanager.h"
#include "/usr/include/unistd.h"
#include <QtCore>
#include <iostream>

Tuner::Tuner(FFTHandler *fft, MicManager *mic, QRect dim, QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::Tuner)
{
    ui->setupUi(this);
    ui->showSignalRadioButton->setVisible(false);
    ui->showSpectreRadioButton->setVisible(false);
    /* The dim parameter set the program size at main */
    noteman = new NoteManager();
    this->setGeometry(dim);

    /* QPalette allows to change the main widget color
       screen*/

    QPalette p = palette();

    p.setColor(QPalette::Background, Qt::white);

    this->setAutoFillBackground(true);
    this->setPalette(p);


    /* Create the spine of the program. Both FFTHandler
       and MicManager are references to main objects */

    fftHandler = fft;
    micManager = mic;

    /* Initial configuration of the plots */

    initializeAxis();
    initializeSpectralPlot();
    initializeSignalPlot();


    /* Setting the interruption period of the timer */

    int timer_interval = 10;
    initializeTimer(timer_interval);

}

void Tuner::initializeAxis(void){

    QVector<double> data_spec_x(512*8);
    QVector<double> data_sig_x(buffer_size);

    for(int i = 0; i < 8*512; i++)
        data_spec_x[i] = i;
    for(int i = 0; i < buffer_size; i++)
        data_sig_x[i] = i;

    this->data_spectre_x = data_spec_x;
    this->data_signal_x = data_sig_x;
}

void Tuner::initializeSpectralPlot(void){

    /* General spectral plot settings */

    ui->spectralPlot->setVisible(false);
    ui->spectralPlot->setBackground(this->palette().background().color());
    ui->spectralPlot->addGraph(); // graph(0) plots the spectre
    ui->spectralPlot->addGraph(); // graph(1) plots the maxima

    /* Graph settings*/

    ui->spectralPlot->graph(0)->setAdaptiveSampling(true);
    ui->spectralPlot->graph(1)->setScatterStyle(QCPScatterStyle(QCPScatterStyle::ssDisc, 10));

    /* Axis settings */

    ui->spectralPlot->xAxis->setRange(0, 512*4);
    ui->spectralPlot->xAxis->setTickLabels(false);
    ui->spectralPlot->xAxis->setLabel("Frecuencia");

    ui->spectralPlot->yAxis->setRange(0, 500);
    ui->spectralPlot->yAxis->setSubTicks(false);
    ui->spectralPlot->yAxis->setTickLabels(false);
    ui->spectralPlot->yAxis->setLabel("Magnitud");

    ui->spectralPlot->xAxis2->setRange(0, 500);
    ui->spectralPlot->xAxis2->setVisible(true);
    ui->spectralPlot->xAxis2->setTickLabels(false);

    ui->spectralPlot->yAxis2->setVisible(true);
    ui->spectralPlot->yAxis2->setTickLabels(false);
    ui->spectralPlot->yAxis2->setSubTicks(false);
}

void Tuner::initializeSignalPlot(void){

    /* General signal plot settings */

    ui->signalPlot->setVisible(false);
    ui->signalPlot->addGraph();
    ui->signalPlot->setBackground(this->palette().background().color());

    /* Graph settings */

    ui->signalPlot->graph(0)->setAdaptiveSampling(true);  

    /* Axis settings */

    ui->signalPlot->xAxis->setRange(0, buffer_size);
    ui->signalPlot->xAxis->grid()->setVisible(false);
    ui->signalPlot->xAxis->setTickLabels(false);
    ui->signalPlot->xAxis->setLabel("Muestras");

    ui->signalPlot->yAxis->setRange(-1000, 1000);
    ui->signalPlot->yAxis->setVisible(true);
    ui->signalPlot->yAxis->setTickLabels(false);
    ui->signalPlot->yAxis->setLabel("Amplitud");
    ui->signalPlot->xAxis2->setRange(0, buffer_size);
    ui->signalPlot->xAxis2->setVisible(true);
    ui->signalPlot->xAxis2->setTickLabels(false);

    ui->signalPlot->yAxis2->setRange(-1000,1000);
    ui->signalPlot->yAxis2->setVisible(true);        
    ui->signalPlot->yAxis2->setTickLabels(false);

}

void Tuner::initializeTimer(int interval){
    QTimer *timer = new QTimer(this);
    connect(timer, SIGNAL(timeout()), this, SLOT(reDrawPlot()));
    timer->start(interval);
}

void Tuner::reDrawPlot(){

    /* Read data from mic and fill the buffer. There are 2 types
       of buffering: the total buffer uses circular buffering to
       obtain a big amount of data without delaying, and the short
       buffer, corresponding to the real-time buffering. Its more
       fast but also its more short */

    micManager->readBuffer();
    short *data_to_fft = micManager->getTotalBuffer();
    short *data_to_plot = micManager->getBuffer();

    /* Compute the DCT of the total buffer, to obtain the spectral
       content of the signal */

    double *p_fft = fftHandler->computeFFT(data_to_fft);

    /* Create 3 types of QVectors. One for the DCT, another for the
       real-time signal, and another for the peaks */
    double maxFreq = dspman->getMaxima(p_fft, 512*8)*8000.0/(512.0*8);

    bool nota_do = false;
    bool nota_re = false;
    bool nota_mi = false;
    bool nota_fa = false;
    bool nota_sol = false;
    bool nota_la = false;
    bool nota_si = false;

    if (/*do*/ 31<maxFreq<33 || 63<maxFreq<66 || 128<maxFreq<132 || 258<maxFreq<264|| 520<maxFreq<526 || 1041<maxFreq<1051 || 2088<maxFreq<2098 || 4180<maxFreq<4190 ){
        nota_do = true;
    }
    if (/*re*/ 34<maxFreq<37 || 71<maxFreq<75 || 144<maxFreq<148 || 290<maxFreq<296 || 584<maxFreq<590 || 1176<maxFreq<1181 || 2344<maxFreq<2354  ){
        nota_re = true;
    }
    if (/*mi*/ 40<maxFreq<42 || 80<maxFreq<84 || 162<maxFreq<166 || 327<maxFreq<331 || 657<maxFreq<661 || 1313<maxFreq<1323 || 2632<maxFreq<2642 ){
        nota_mi = true;
    }
    if (/*fa*/ 42<maxFreq<44 || 85<maxFreq<89 || 172<maxFreq<176 || 347<maxFreq<351 || 695<maxFreq<701 || 1391<maxFreq<1401 || 2788<maxFreq<2798  ){
        nota_fa = true;
    }
    if (/*sol*/ 47<maxFreq<49 || 95<maxFreq<99 || 193<maxFreq<197 || 387<maxFreq<294 || 780<maxFreq<786 || 1562<maxFreq<1572 || 3130<maxFreq<3140 ){
        nota_sol = true;
    }
    if (/*la*/ 54<maxFreq<56 ||108<maxFreq<112 || 218<maxFreq<222 || 437<maxFreq<443 || 877<maxFreq<883 || 1755<maxFreq<1765 || 3515<maxFreq<3525 ){
        nota_la = true;
    }
    if (/*si*/ 60<maxFreq<62 ||121<maxFreq<125 || 244<maxFreq<248 || 490<maxFreq<496 || 984<maxFreq<990 || 1970<maxFreq<1980 || 3946<maxFreq<3956 ){
        nota_si = true;
    }

    if(nota_do || nota_re || nota_mi || nota_fa || nota_sol || nota_la || nota_si){
        ui->maxFreqNumberAfinado->display(maxFreq);
    }
    else ui->maxFreqNumber->display(maxFreq);

    QVector<double> data_fft(8*512);
    QVector<double> data_signal(buffer_size);
    QVector<double> data_fft_max(8*512);
    data_fft_max.fill(0);

    /* Fill the new data and add to the properties */

    for(int i = 0; i < 512*8; i++)
        data_fft[i] = p_fft[i];

    for(int i = 0; i<buffer_size; i++)
        data_signal[i] = data_to_plot[i]/16;

    this->data_spectre_y = data_fft;
    this->data_spectre_y_max = data_fft_max;
    this->data_signal_y = data_signal;

    /* Replot the graphs with new data */

    ui->spectralPlot->graph(0)->setData(this->data_spectre_x,this->data_spectre_y);
    //ui->spectralPlot->graph(1)->setData(this->data_spectre_x, this->data_spectre_y_max);
    ui->signalPlot->graph(0)->setData(this->data_signal_x,this->data_signal_y);
    ui->spectralPlot->replot();
    ui->signalPlot->replot();

    QString note = QString::fromStdString(noteman->getNearNote(maxFreq));
    ui->noteLabel->setText(note);

    short cent = noteman->getCentPitch(maxFreq);
    if(cent > 0){
        ui->posCentBar->setValue(cent);
        ui->negCentBar->setValue(0);
    }
    else{
        ui->negCentBar->setValue(cent);
        ui->posCentBar->setValue(0);
    }

}

/* ------------------------------------------------------------------ */
/* --------------------------- EVENTS ------------------------------- */
/* ------------------------------------------------------------------ */

void Tuner::on_showSpectreRadioButton_clicked(bool checked)
{
    if(checked){
        QRect geom = this->geometry();
        geom.setHeight(481);
        this->setGeometry(geom);
        ui->spectralPlot->setVisible(checked);
        ui->signalPlot->setVisible(!checked);
    }
}

void Tuner::on_showSignalRadioButton_clicked(bool checked)
{
    if(checked){
        QRect geom = this->geometry();
        geom.setHeight(481);
        this->setGeometry(geom);
        ui->signalPlot->setVisible(checked);
        ui->spectralPlot->setVisible(!checked);
    }
}

void Tuner::on_action64_triggered(bool checked)
{
    if(checked){
        buffer_size = 64;
        micManager->setBufferSize(buffer_size);
        fftHandler->changeFFTPoints(buffer_size);
    }
}

void Tuner::on_checkBox_clicked(bool checked)
{
    ui->showSignalRadioButton->setVisible(checked);
    ui->showSpectreRadioButton->setVisible(checked);
    ui->spectralPlot->setVisible(checked);
    ui->signalPlot->setVisible(checked);
    ui->maxFreqNumber->setVisible(checked);
    ui->maxFreqNumberAfinado->setVisible(checked);

    this->geometry();
    if(checked)
        this->setGeometry(this->x(),this->y(),683,514);
    else
        this->setGeometry(this->x(),this->y(),683,114);
}
